/************************************************************************/
/*																		*/
/*	SFSPI.cpp	--	Definition for SFSPI library 	    				*/
/*																		*/
/************************************************************************/
/*	Author:		Michelle Yu												*/
/*	Copyright 2011, Digilent Inc.										*/
/************************************************************************/
/*  File Description:													*/
/*		This file defines functions for SFSPI							*/
/*																		*/
/************************************************************************/
/*  Revision History:													*/
/*																		*/
/*	09/09/2011(MichelleY): created										*/
/*																		*/
/************************************************************************/


/* ------------------------------------------------------------ */
/*				Include File Definitions						*/
/* ------------------------------------------------------------ */
#include "SFSPI.h"



/* ------------------------------------------------------------ */
/*				Procedure Definitions							*/
/* ------------------------------------------------------------ *

/* ------------------------------------------------------------------- */
/** void SFSPI::begin()
**
**	Parameters:
**		None
**
**
**	Return Value:
**		None
**
**	Errors:
**		TBD
**
**	Description:
**		This function setup SPI 
*/
void SFSPI::begin()
{       
    
    //Turn on SPI
    mySPI.begin();
    
}


/* ------------------------------------------------------------------- */
/** int SFSPI::getDeviceID()
**
**	Parameters:
**		None
**
**
**	Return Value:
**		Returns Chip Model 
				0x16  - M25P16
				0x128 - M25P128
				0 	  - if anything else
**
**	Errors:
**		TBD
**
**	Description:
**		This function returns the device ID
*/
int SFSPI::getDeviceID()
{

	uint8_t rcvByte[3];
	
	// wait for any unfinished write or erase cycle
	// this is called as that command will not be ignored during a write or erase cycle
	this->waitForWrite();

    
    mySPI.setSelect(LOW);
    
	//set command  0x9F for Read ID
	mySPI.transfer(0x9F);
	
    //Get device ID
    rcvByte[0] = mySPI.transfer(0x00);
    rcvByte[1] = mySPI.transfer(0x00);
    rcvByte[2] = mySPI.transfer(0x00);
    
    mySPI.setSelect(HIGH);
    
 
	if(rcvByte[2] == 0x15)
	{
		return 16;
	}
	else if(rcvByte[2] == 0x18)
	{
		return 128;	
	}
	else
	{
		return 0;
	}
    
}

/* ------------------------------------------------------------------- */
/** uint8_t  SFSPI::normalRead(int address)
**
**	Parameters:
**		address		- a 3 byte address to read from
**
**	Return Value:
**		Returns a byte read
**
**	Errors:
**		TBD
**
**	Description:
**		This function reads a byte of data from the address specified
*/
uint8_t  SFSPI::normalRead(int address)
{
	uint8_t sndByte[4]={0,0,0,0};
	uint8_t rcvByte;
	
	//set command  0x03 for Read at normal speed 
	sndByte[0] = 0x03;
	//set 3 byte address MSB first 
	sndByte[1] = (address >> 16);
	sndByte[2] = (address >> 8);
	sndByte[3] =  address;	
	

	// wait for any unfinished write or erase cycle
	// this is called as that command will not be ignored during a write or erase cycle
	this->waitForWrite();
	
	mySPI.setSelect(LOW);
    
	mySPI.transfer(sndByte[0]);
    mySPI.transfer(sndByte[1]);
    mySPI.transfer(sndByte[2]);
    mySPI.transfer(sndByte[3]);
    
    //read a byte 	
    rcvByte = mySPI.transfer(0x00);
       
    mySPI.setSelect(HIGH);
	
	return rcvByte;	

}

/* ------------------------------------------------------------------- */
/**uint8_t  SFSPI::fastRead(int address)
**
**	Parameters:
**		address		- a 3 byte address to read from
**
**	Return Value:
**		Returns a byte read
**
**	Errors:
**		TBD
**
**	Description:
**		This function reads a byte of data from the address specified
** at a faster speed
*/
uint8_t  SFSPI::fastRead(int address)
{
	uint8_t sndByte[5]={0,0,0,0,0};
	uint8_t rcvByte;
	
	//set command  0x0B for Read at fast speed 
	sndByte[0] = 0x0B;
	//set 3 byte address MSB first 
	sndByte[1] = (address >> 16);
	sndByte[2] = (address >> 8);
	sndByte[3] = address;
	sndByte[4] = 0x00;


	// wait for any unfinished write or erase cycle
	// this is called as that command will not be ignored during a write or erase cycle
	this->waitForWrite();
	
	mySPI.setSelect(LOW);
    
	mySPI.transfer(sndByte[0]);
    mySPI.transfer(sndByte[1]);
    mySPI.transfer(sndByte[2]);
    mySPI.transfer(sndByte[3]);
    mySPI.transfer(sndByte[4]);
    
    //read a byte 	
    rcvByte = mySPI.transfer(0x00);
       
    mySPI.setSelect(HIGH);
	
	return rcvByte;	

}


/* ------------------------------------------------------------------- */
/** void  SFSPI::writePage(uint8_t writeData, int address)
**
**	Parameters:
**		address		- a 3 byte address to write to 
**		writeData	- a byte of data to write	
**
**	Return Value:
**		none
**
**	Errors:
**		TBD
**
**	Description:
**		This function writes a Byte of data to the address specified 
**		Write cycle begins after the function set SS high
*/
void  SFSPI::writePage(uint8_t writeData, int address)
{
	uint8_t sndByte[5]={0,0,0,0,0};

	//set command  0x02 for Page Program 
	sndByte[0] = 0x02;
	//set 3 byte address MSB first 
	sndByte[1] = (address >> 16);
	sndByte[2] = (address >> 8);
	sndByte[3] = (address);
	sndByte[4] = writeData;
	
	

	// wait for any unfinished write or erase cycle
	// this is called as that command will not be ignored during a write or erase cycle
	this->waitForWrite();
	
	//enable write
	this->writeEnable();
	
	mySPI.setSelect(LOW);
    
	mySPI.transfer(sndByte[0]);
    mySPI.transfer(sndByte[1]);
    mySPI.transfer(sndByte[2]);
    mySPI.transfer(sndByte[3]);
    mySPI.transfer(sndByte[4]);
       
    mySPI.setSelect(HIGH);
	
}

/* ------------------------------------------------------------------- */
/** void  SFSPI::sectorErase(int address)
**
**	Parameters:
**		address		- a 3 byte address in the sector to be erased
**		
**
**	Return Value:
**		none
**
**	Errors:
**		TBD
**
**	Description:
**		This function erases the sector that the address is in
**		Erase cycle begins after the function set SS high
*/
void  SFSPI::sectorErase(int address)
{
	uint8_t sndByte[4]={0,0,0,0};
	
	//set command  0xD8 for sector erase 
	sndByte[0] = 0xD8;
	//set 3 byte address MSB first 
	sndByte[1] = (address >> 16);
	sndByte[2] = (address >> 8);
	sndByte[3] = (address);

	// wait for any unfinished write or erase cycle
	// this is called as that command will not be ignored during a write or erase cycle
	this->waitForWrite();
	
	//enable write
	this->writeEnable();
	
	mySPI.setSelect(LOW);
    
	mySPI.transfer(sndByte[0]);
    mySPI.transfer(sndByte[1]);
    mySPI.transfer(sndByte[2]);
    mySPI.transfer(sndByte[3]);
       
    mySPI.setSelect(HIGH);

}

/* ------------------------------------------------------------------- */
/**void  SFSPI::bulkErase()
**
**	Parameters:
**		None
**
**	Return Value:
**		None
**
**	Errors:
**		TBD
**
**	Description:
**		This function erases the entire memory
**		Erase cycle begins after the function set SS high
*/
void  SFSPI::bulkErase()
{

	// wait for any unfinished write or erase cycle
	// this is called as that command will not be ignored during a write or erase cycle
	this->waitForWrite();
	this->writeSR(0x00);
	//enable write
	this->writeEnable();

	mySPI.setSelect(LOW);

	//set command  0xC7 for bulk erase 
	mySPI.transfer(0xC7);
    
    mySPI.setSelect(HIGH);

}

/* ------------------------------------------------------------------- */
/**void  SFSPI::writeSR(uint8_t setReg)
**
**	Parameters:
**		setReg		- the value to set the register with
**
**	Return Value:
**		none
**
**	Errors:
**		TBD
**
**	Description:
//	This function writes to the status register. 
    It can be use to protect certain sectors against write/erase with the corresponding 
    values shown below.
  	ChipModel can be find with GetDeviceID();

	Device ID 16:
		0x00 none 
		0x04 Upper 32nd (Sector 31) 
		0x08 Upper sixteenth (two sectors: 30 and 31)
		0x0C Upper eighth (four sectors: 28 to 31) 
		0x10 Upper quarter (eight sectors: 24 to 31) 
		0x14 Upper half (sixteen sectors: 16 to 31) 
		0x18 All sectors (32 sectors: 0 to 31) 
		0x1C All sectors (32 sectors: 0 to 31) 
	Device ID 128:
		0x00 none 
		0x04 Upper 64th (1 Sector, 2Mb) 
		0x08 Upper 32nd (2 Sectors, 4Mb) 
		0x0C Upper 16nd (4 Sectors, 8Mb) 
		0x10 Upper 8nd (8 Sectors, 16Mb) 
		0x14 Upper Quarter (16 Sectors, 32Mb) 
		0x18 Upper Half (32 Sectors, 64Mb) 
		0x1C All sectors (64 Sectors, 128Mb) 
*/
void  SFSPI::writeSR(uint8_t setReg)
{
	// wait for any unfinished write or erase cycle
	// this is called as that command will not be ignored during a write or erase cycle
	this->waitForWrite();
	
	//enable write
	this->writeEnable();

	mySPI.setSelect(LOW);

	//set command  0x01 to Write to Status Register
	mySPI.transfer(0x01);
	mySPI.transfer(setReg);
    
    mySPI.setSelect(HIGH);
}
/*--------------------------------------------------------------*/
/** void  SFSPI::writeEnable()
**
**	Parameters:
**	    None
**
**	Return Value:
**		None
**
**	Errors:
**		TBD
**
**	Description:
**		This function enables the device to write or erase
*/
void SFSPI::writeEnable()
{
	mySPI.setSelect(LOW);

	//set command to 0x06 for write enable
	mySPI.transfer(0x06);
    
    mySPI.setSelect(HIGH);

}

/* ------------------------------------------------------------------- */
/** uint8_t SFSPI::readSR()
**
**	Parameters:
**		none
**
**	Return Value:
**		Returns status register
**
**	Errors:
**		TBD
**
**	Description:
**		This function returns the status register
*/
uint8_t SFSPI::readSR()
{
	uint8_t SR;
	
    mySPI.setSelect(LOW);
    
	//set command  0x05 for read status register 
	mySPI.transfer(0x05);
    //Get status register
    SR = mySPI.transfer(0x00);
    
    mySPI.setSelect(HIGH);

	return SR;	

}

/* ------------------------------------------------------------------- */
/** void  SFSPI::waitForWrite()
**
**	Parameters:
**		none
**
**	Return Value:
**		none
**
**	Errors:
**		TBD
**
**	Description:
**		This function waits for memory to be finish with a write or erase cycle	
*/
void  SFSPI::waitForWrite()
{
	uint8_t SR;

    SR = this->readSR();
	
	//0x1 means a write is in progess
	while(SR & 0x1)
	{
		//keep polling status register
		SR = this->readSR();
	}
	
}